@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2006 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -32,7 +32,7 @@
 
 #include <locale.h>
 
-#if defined(ENABLE_NLS) && !defined(_)
+#if ENABLE_NLS == 1 && !defined(_)
     #include <libintl.h>
     #define _(string) gettext(string)
     #define NG_(single,plural,number) ngettext(single,plural,number)
@@ -41,16 +41,17 @@
     #else
         #define N_(string) (string)
     #endif /* gettext_noop */
-#endif /* defined(ENABLE_NLS) && !defined(_) */
+#endif /* ENABLE_NLS == 1 && !defined(_) */
 #if !defined(_)
     #define _(string) (string)
     #define NG_(single,plural,number) ((number == 1) ? single : plural)
     #define N_(string) (string)
     #define gettext(string) (string)
 #endif /* !defined(_) */
+#define AI(string) (string)
 
 
-#define WEECHAT_COPYRIGHT_DATE   "(C) 2003-2023"
+#define WEECHAT_COPYRIGHT_DATE   "(C) 2003-2024"
 #define WEECHAT_WEBSITE          "https://weechat.org/"
 #define WEECHAT_WEBSITE_DOWNLOAD "https://weechat.org/download/"
 
@@ -114,6 +115,7 @@ extern int weechat_home_temp;
 extern int weechat_home_delete_on_exit;
 extern char *weechat_config_dir;
 extern char *weechat_data_dir;
+extern char *weechat_state_dir;
 extern char *weechat_cache_dir;
 extern char *weechat_runtime_dir;
 extern char *weechat_local_charset;
