@@ -1,7 +1,7 @@
 /*
  * alias-command.c - alias commands
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -33,7 +33,7 @@
 
 void
 alias_command_add (const char *alias_name, const char *command,
-                   const char *completion)
+                   const char *completion, int update)
 {
     struct t_config_option *ptr_option;
 
@@ -52,22 +52,22 @@ alias_command_add (const char *alias_name, const char *command,
     ptr_option = weechat_config_search_option (alias_config_file,
                                                alias_config_section_cmd,
                                                alias_name);
-    if (ptr_option)
-        weechat_config_option_free (ptr_option);
+    weechat_config_option_free (ptr_option);
     alias_config_cmd_new_option (alias_name, command);
 
     /* create configuration option for completion */
     ptr_option = weechat_config_search_option (alias_config_file,
                                                alias_config_section_completion,
                                                alias_name);
-    if (ptr_option)
-        weechat_config_option_free (ptr_option);
+    weechat_config_option_free (ptr_option);
     if (completion)
         alias_config_completion_new_option (alias_name, completion);
 
     /* display message */
     weechat_printf (NULL,
-                    _("Alias \"%s\" => \"%s\" created"),
+                    (update) ?
+                    _("Alias updated: \"%s\" => \"%s\"") :
+                    _("Alias created: \"%s\" => \"%s\""),
                     alias_name, command);
 }
 
@@ -80,10 +80,10 @@ alias_command_cb (const void *pointer, void *data,
                   struct t_gui_buffer *buffer, int argc,
                   char **argv, char **argv_eol)
 {
-    char *ptr_alias_name, *ptr_alias_name2;
-    struct t_alias *ptr_alias, *ptr_alias2;
+    char *ptr_alias_name, *ptr_alias_name2, *name;
+    struct t_alias *ptr_alias, *ptr_alias2, *ptr_next_alias;
     struct t_config_option *ptr_option;
-    int alias_found, i;
+    int alias_found, i, update;
 
     /* make C compiler happy */
     (void) pointer;
@@ -171,65 +171,111 @@ alias_command_cb (const void *pointer, void *data,
         return WEECHAT_RC_OK;
     }
 
-    if (weechat_strcmp (argv[1], "add") == 0)
+    if ((weechat_strcmp (argv[1], "add") == 0)
+        || (weechat_strcmp (argv[1], "addreplace") == 0))
     {
-        WEECHAT_COMMAND_MIN_ARGS(4, "add");
-        alias_command_add (
-            (weechat_string_is_command_char (argv[2])) ?
-            (char *)weechat_utf8_next_char (argv[2]) : argv[2],
-            argv_eol[3],
-            NULL);
+        WEECHAT_COMMAND_MIN_ARGS(4, argv[1]);
+        update = 0;
+        ptr_alias_name = (weechat_string_is_command_char (argv[2])) ?
+            (char *)weechat_utf8_next_char (argv[2]) : argv[2];
+        ptr_alias = alias_search (ptr_alias_name);
+        if (ptr_alias)
+        {
+            if (weechat_strcmp (argv[1], "addreplace") == 0)
+            {
+                alias_free (ptr_alias);
+                ptr_option = weechat_config_search_option (
+                    alias_config_file,
+                    alias_config_section_cmd,
+                    ptr_alias_name);
+                weechat_config_option_free (ptr_option);
+                ptr_option = weechat_config_search_option (
+                    alias_config_file,
+                    alias_config_section_completion,
+                    ptr_alias_name);
+                weechat_config_option_free (ptr_option);
+                update = 1;
+            }
+            else
+            {
+                weechat_printf (NULL,
+                                _("%sAlias \"%s\" already exists"),
+                                weechat_prefix ("error"),
+                                ptr_alias_name);
+                return WEECHAT_RC_ERROR;
+            }
+        }
+        alias_command_add (ptr_alias_name, argv_eol[3], NULL, update);
         return WEECHAT_RC_OK;
     }
 
-    if (weechat_strcmp (argv[1], "addcompletion") == 0)
+    if ((weechat_strcmp (argv[1], "addcompletion") == 0)
+        || (weechat_strcmp (argv[1], "addreplacecompletion") == 0))
     {
-        WEECHAT_COMMAND_MIN_ARGS(5, "addcompletion");
-        alias_command_add (
-            (weechat_string_is_command_char (argv[3])) ?
-            (char *)weechat_utf8_next_char (argv[3]) : argv[3],
-            argv_eol[4],
-            argv[2]);
+        WEECHAT_COMMAND_MIN_ARGS(5, argv[1]);
+        update = 0;
+        ptr_alias_name = (weechat_string_is_command_char (argv[3])) ?
+            (char *)weechat_utf8_next_char (argv[3]) : argv[3];
+        ptr_alias = alias_search (ptr_alias_name);
+        if (ptr_alias)
+        {
+            if (weechat_strcmp (argv[1], "addreplacecompletion") == 0)
+            {
+                alias_free (ptr_alias);
+                ptr_option = weechat_config_search_option (
+                    alias_config_file,
+                    alias_config_section_cmd,
+                    ptr_alias_name);
+                weechat_config_option_free (ptr_option);
+                ptr_option = weechat_config_search_option (
+                    alias_config_file,
+                    alias_config_section_completion,
+                    ptr_alias_name);
+                weechat_config_option_free (ptr_option);
+                update = 1;
+            }
+            else
+            {
+                weechat_printf (NULL,
+                                _("%sAlias \"%s\" already exists"),
+                                weechat_prefix ("error"),
+                                ptr_alias_name);
+                return WEECHAT_RC_ERROR;
+            }
+        }
+        alias_command_add (ptr_alias_name, argv_eol[4], argv[2], update);
         return WEECHAT_RC_OK;
     }
 
     if (weechat_strcmp (argv[1], "del") == 0)
     {
-        WEECHAT_COMMAND_MIN_ARGS(3, "del");
+        WEECHAT_COMMAND_MIN_ARGS(3, argv[1]);
         for (i = 2; i < argc; i++)
         {
             ptr_alias_name = (weechat_string_is_command_char (argv[i])) ?
                 (char *)weechat_utf8_next_char (argv[i]) : argv[i];
-            ptr_alias = alias_search (ptr_alias_name);
-            if (!ptr_alias)
+            ptr_alias = alias_list;
+            while (ptr_alias)
             {
-                weechat_printf (NULL,
-                                _("%sAlias \"%s\" not found"),
-                                weechat_prefix ("error"),
-                                ptr_alias_name);
-            }
-            else
-            {
-                /* remove alias */
-                alias_free (ptr_alias);
-
-                /* remove options */
-                ptr_option = weechat_config_search_option (
-                    alias_config_file,
-                    alias_config_section_cmd,
-                    ptr_alias_name);
-                if (ptr_option)
+                ptr_next_alias = ptr_alias->next_alias;
+                if (weechat_string_match (ptr_alias->name, ptr_alias_name, 1))
+                {
+                    name = strdup (ptr_alias->name);
+                    alias_free (ptr_alias);
+                    ptr_option = weechat_config_search_option (
+                        alias_config_file,
+                        alias_config_section_cmd,
+                        ptr_alias_name);
                     weechat_config_option_free (ptr_option);
-                ptr_option = weechat_config_search_option (
-                    alias_config_file,
-                    alias_config_section_completion,
-                    ptr_alias_name);
-                if (ptr_option)
+                    ptr_option = weechat_config_search_option (
+                        alias_config_file,
+                        alias_config_section_completion,
+                        ptr_alias_name);
                     weechat_config_option_free (ptr_option);
-
-                weechat_printf (NULL,
-                                _("Alias \"%s\" removed"),
-                                ptr_alias_name);
+                    weechat_printf (NULL, _("Alias \"%s\" removed"), name);
+                    free (name);
+                }
+                ptr_alias = ptr_next_alias;
             }
         }
         return WEECHAT_RC_OK;
@@ -237,7 +283,7 @@ alias_command_cb (const void *pointer, void *data,
 
     if (weechat_strcmp (argv[1], "rename") == 0)
     {
-        WEECHAT_COMMAND_MIN_ARGS(4, "rename");
+        WEECHAT_COMMAND_MIN_ARGS(4, argv[1]);
 
         ptr_alias_name = (weechat_string_is_command_char (argv[2])) ?
             (char *)weechat_utf8_next_char (argv[2]) : argv[2];
@@ -251,6 +297,7 @@ alias_command_cb (const void *pointer, void *data,
                             _("%sAlias \"%s\" not found"),
                             weechat_prefix ("error"),
                             ptr_alias_name);
+            return WEECHAT_RC_ERROR;
         }
         else
         {
@@ -260,6 +307,7 @@ alias_command_cb (const void *pointer, void *data,
             {
                 weechat_printf (NULL, _("%sAlias \"%s\" already exists"),
                                 weechat_prefix ("error"), ptr_alias_name2);
+                return WEECHAT_RC_ERROR;
             }
             else
             {
@@ -285,7 +333,8 @@ alias_command_cb (const void *pointer, void *data,
             {
                 alias_command_add (alias_default[i][0],  /* name */
                                    alias_default[i][1],  /* command */
-                                   alias_default[i][2]);  /* completion */
+                                   alias_default[i][2],  /* completion */
+                                   0);  /* update */
             }
         }
         return WEECHAT_RC_OK;
@@ -305,54 +354,54 @@ alias_command_init ()
         "alias",
         N_("list, add or remove command aliases"),
         /* TRANSLATORS: only text between angle brackets (eg: "<name>") must be translated */
-        N_("list [<alias>]"
-           " || add <alias> [<command>[;<command>...]]"
-           " || addcompletion <completion> <alias> [<command>[;<command>...]]"
-           " || del <alias> [<alias>...]"
-           " || rename <alias> <new_alias>"
+        N_("list [<name>]"
+           " || add|addreplace <name> [<command>[;<command>...]]"
+           " || addcompletion|addreplacecompletion <completion> <name> "
+           "[<command>[;<command>...]]"
+           " || del <name>|<mask> [<name>|<mask>...]"
+           " || rename <name> <new_name>"
            " || missing"),
         /* xgettext:no-c-format */
-        N_("         list: list aliases (without argument, this list is "
-           "displayed)\n"
-           "          add: add an alias\n"
-           "addcompletion: add an alias with a custom completion\n"
-           "          del: delete an alias\n"
-           "       rename: rename an alias\n"
-           "      missing: add missing aliases (using default aliases)\n"
-           "   completion: completion for alias: by default completion is "
-           "done with target command\n"
-           "               note: you can use %%command to use completion of "
-           "an existing command\n"
-           "        alias: name of alias\n"
-           "      command: command name with arguments (many commands can be "
-           "separated by semicolons)\n"
-           "\n"
-           "Note: in command, special variables are replaced:\n"
-           "        $n: argument 'n' (between 1 and 9)\n"
-           "       $-m: arguments from 1 to 'm'\n"
-           "       $n-: arguments from 'n' to last\n"
-           "      $n-m: arguments from 'n' to 'm'\n"
-           "        $*: all arguments\n"
-           "        $&: all arguments, with \" replaced by \\\"\n"
-           "        $~: last argument\n"
-           "      $var: where \"var\" is a local variable of buffer (see "
-           "/buffer listvar)\n"
-           "            examples: $nick, $channel, $server, $plugin, $name\n"
-           "\n"
-           "Examples:\n"
-           "  alias /split to split window horizontally:\n"
-           "    /alias add split /window splith\n"
-           "  alias /hello to say \"hello\" on all channels but not on "
-           "#weechat:\n"
-           "    /alias add hello /allchan -exclude=#weechat hello\n"
-           "  rename alias \"hello\" to \"Hello\":\n"
-           "    /alias rename hello Hello\n"
-           "  alias /forcejoin to send IRC command \"forcejoin\" with "
-           "completion of /sajoin:\n"
-           "    /alias addcompletion %%sajoin forcejoin /quote forcejoin"),
+        WEECHAT_CMD_ARGS_DESC(
+            N_("raw[list]: list aliases (without argument, this list is "
+               "displayed)"),
+            N_("raw[add]: add an alias"),
+            N_("raw[addreplace]: add or replace an existing alias"),
+            N_("raw[addcompletion]: add an alias with a custom completion"),
+            N_("raw[addreplacecompletion]: add or replace an existing alias "
+               "with a custom completion"),
+            N_("name: name of alias"),
+            N_("completion: completion for alias: by default completion is "
+               "done with target command (you can use \"%%command\" to use the "
+               "completion of an existing command)"),
+            N_("command: command name with arguments (many commands can be "
+               "separated by semicolons)"),
+            N_("raw[del]: delete aliases"),
+            N_("mask: name where wildcard \"*\" is allowed"),
+            N_("raw[rename]: rename an alias"),
+            N_("raw[missing]: add missing aliases (using default aliases)"),
+            "",
+            N_("In command, special variables are replaced:"),
+            N_("  $n: argument \"n\" (between 1 and 9)"),
+            N_("  $-m: arguments from 1 to \"m\""),
+            N_("  $n-: arguments from \"n\" to last"),
+            N_("  $n-m: arguments from \"n\" to \"m\""),
+            N_("  $*: all arguments"),
+            N_("  $&: all arguments, with \" replaced by \\\""),
+            N_("  $~: last argument"),
+            N_("  $var: where \"var\" is a local variable of buffer (see "
+               "/buffer listvar), examples: $nick, $channel, $server, $plugin, "
+               "$name"),
+            "",
+            N_("Examples:"),
+            AI("  /alias add split /window splith"),
+            AI("  /alias add hello /allchan -exclude=#weechat hello"),
+            AI("  /alias rename hello Hello"),
+            AI("  /alias addcompletion %%sajoin forcejoin /quote forcejoin")),
         "list %(alias)"
-        " || add %(alias) %(commands:/)|%(alias_value)"
-        " || addcompletion %- %(alias) %(commands:/)|%(alias_value)"
+        " || add|addreplace %(alias) %(commands:/)|%(alias_value)"
+        " || addcompletion|addreplacecompletion %- %(alias) "
+        "%(commands:/)|%(alias_value)"
         " || del %(alias)|%*"
         " || rename %(alias) %(alias)"
         " || missing",
