@@ -677,7 +677,7 @@ exec_command_exec (const void *pointer, void *data,
     /* send text to a running process */
     if (weechat_strcmp (argv[1], "-in") == 0)
     {
-        WEECHAT_COMMAND_MIN_ARGS(4, "-in");
+        WEECHAT_COMMAND_MIN_ARGS(4, argv[1]);
         ptr_exec_cmd = exec_command_search_running_id (argv[2]);
         if (ptr_exec_cmd && ptr_exec_cmd->hook)
         {
@@ -696,7 +696,7 @@ exec_command_exec (const void *pointer, void *data,
     /* send text to a running process (if given), then close stdin */
     if (weechat_strcmp (argv[1], "-inclose") == 0)
     {
-        WEECHAT_COMMAND_MIN_ARGS(3, "-inclose");
+        WEECHAT_COMMAND_MIN_ARGS(3, argv[1]);
         ptr_exec_cmd = exec_command_search_running_id (argv[2]);
         if (ptr_exec_cmd && ptr_exec_cmd->hook)
         {
@@ -719,7 +719,7 @@ exec_command_exec (const void *pointer, void *data,
     /* send a signal to a running process */
     if (weechat_strcmp (argv[1], "-signal") == 0)
     {
-        WEECHAT_COMMAND_MIN_ARGS(4, "-signal");
+        WEECHAT_COMMAND_MIN_ARGS(4, argv[1]);
         ptr_exec_cmd = exec_command_search_running_id (argv[2]);
         if (ptr_exec_cmd)
             weechat_hook_set (ptr_exec_cmd->hook, "signal", argv[3]);
@@ -729,7 +729,7 @@ exec_command_exec (const void *pointer, void *data,
     /* send a KILL signal to a running process */
     if (weechat_strcmp (argv[1], "-kill") == 0)
     {
-        WEECHAT_COMMAND_MIN_ARGS(3, "-kill");
+        WEECHAT_COMMAND_MIN_ARGS(3, argv[1]);
         ptr_exec_cmd = exec_command_search_running_id (argv[2]);
         if (ptr_exec_cmd)
             weechat_hook_set (ptr_exec_cmd->hook, "signal", "kill");
@@ -753,7 +753,7 @@ exec_command_exec (const void *pointer, void *data,
     /* set a hook property */
     if (weechat_strcmp (argv[1], "-set") == 0)
     {
-        WEECHAT_COMMAND_MIN_ARGS(5, "-set");
+        WEECHAT_COMMAND_MIN_ARGS(5, argv[1]);
         ptr_exec_cmd = exec_command_search_running_id (argv[2]);
         if (ptr_exec_cmd)
             weechat_hook_set (ptr_exec_cmd->hook, argv[3], argv_eol[4]);
@@ -763,7 +763,7 @@ exec_command_exec (const void *pointer, void *data,
     /* delete terminated command(s) */
     if (weechat_strcmp (argv[1], "-del") == 0)
     {
-        WEECHAT_COMMAND_MIN_ARGS(3, "-del");
+        WEECHAT_COMMAND_MIN_ARGS(3, argv[1]);
         if (weechat_strcmp (argv[2], "-all") == 0)
         {
             count = 0;
