@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -23,7 +23,7 @@
 #define IRC_CONFIG_NAME "irc"
 #define IRC_CONFIG_PRIO_NAME (TO_STR(IRC_PLUGIN_PRIORITY) "|" IRC_CONFIG_NAME)
 
-#define IRC_CONFIG_VERSION 3
+#define IRC_CONFIG_VERSION 5
 
 enum t_irc_config_look_server_buffer
 {
@@ -112,6 +112,7 @@ extern struct t_config_option *irc_config_look_display_host_join;
 extern struct t_config_option *irc_config_look_display_host_join_local;
 extern struct t_config_option *irc_config_look_display_host_notice;
 extern struct t_config_option *irc_config_look_display_host_quit;
+extern struct t_config_option *irc_config_look_display_host_wallops;
 extern struct t_config_option *irc_config_look_display_join_message;
 extern struct t_config_option *irc_config_look_display_old_topic;
 extern struct t_config_option *irc_config_look_display_pv_away_once;
@@ -122,11 +123,14 @@ extern struct t_config_option *irc_config_look_highlight_channel;
 extern struct t_config_option *irc_config_look_highlight_pv;
 extern struct t_config_option *irc_config_look_highlight_server;
 extern struct t_config_option *irc_config_look_highlight_tags_restrict;
+extern struct t_config_option *irc_config_look_ignore_tag_messages;
 extern struct t_config_option *irc_config_look_item_channel_modes_hide_args;
 extern struct t_config_option *irc_config_look_item_display_server;
 extern struct t_config_option *irc_config_look_item_nick_modes;
 extern struct t_config_option *irc_config_look_item_nick_prefix;
 extern struct t_config_option *irc_config_look_join_auto_add_chantype;
+extern struct t_config_option *irc_config_look_list_buffer;
+extern struct t_config_option *irc_config_look_list_buffer_format_export;
 extern struct t_config_option *irc_config_look_list_buffer_scroll_horizontal;
 extern struct t_config_option *irc_config_look_list_buffer_sort;
 extern struct t_config_option *irc_config_look_list_buffer_topic_strip_colors;
@@ -165,7 +169,6 @@ extern struct t_config_option *irc_config_look_temporary_servers;
 extern struct t_config_option *irc_config_look_topic_strip_colors;
 
 extern struct t_config_option *irc_config_color_input_nick;
-extern struct t_config_option *irc_config_color_item_channel_modes;
 extern struct t_config_option *irc_config_color_item_lag_counting;
 extern struct t_config_option *irc_config_color_item_lag_finished;
 extern struct t_config_option *irc_config_color_item_nick_modes;
@@ -185,6 +188,7 @@ extern struct t_config_option *irc_config_color_nick_prefixes;
 extern struct t_config_option *irc_config_color_notice;
 extern struct t_config_option *irc_config_color_reason_kick;
 extern struct t_config_option *irc_config_color_reason_quit;
+extern struct t_config_option *irc_config_color_term_remap;
 extern struct t_config_option *irc_config_color_topic_current;
 extern struct t_config_option *irc_config_color_topic_new;
 extern struct t_config_option *irc_config_color_topic_old;
@@ -210,6 +214,7 @@ extern struct t_config_option *irc_config_server_default[];
 extern struct t_hashtable *irc_config_hashtable_display_join_message;
 extern struct t_hashtable *irc_config_hashtable_nick_prefixes;
 extern struct t_hashtable *irc_config_hashtable_color_mirc_remap;
+extern struct t_hashtable *irc_config_hashtable_color_term_remap;
 extern char **irc_config_nicks_hide_password;
 extern int irc_config_num_nicks_hide_password;
 
