@@ -1,7 +1,7 @@
 /*
  * irc-input.c - input data management for IRC buffers
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -64,6 +64,8 @@
 void
 irc_input_user_message_display (struct t_irc_server *server,
                                 time_t date,
+                                int date_usec,
+                                struct t_hashtable *tags,
                                 const char *target,
                                 const char *address,
                                 const char *command,
@@ -74,6 +76,7 @@ irc_input_user_message_display (struct t_irc_server *server,
     struct t_irc_channel *ptr_channel;
     struct t_gui_buffer *ptr_buffer;
     struct t_irc_nick *ptr_nick;
+    struct t_irc_protocol_ctxt ctxt;
     const char *ptr_target;
     char *text2, *text_decoded, str_tags[256], *str_color;
     const char *ptr_text;
@@ -82,6 +85,14 @@ irc_input_user_message_display (struct t_irc_server *server,
     if (!server || !target)
         return;
 
+    memset (&ctxt, 0, sizeof (ctxt));
+    ctxt.server = server;
+    ctxt.date = date;
+    ctxt.date_usec = date_usec;
+    ctxt.tags = tags;
+    ctxt.address = (char *)address;
+    ctxt.command = (char *)command;
+
     is_notice = (weechat_strcasecmp (command, "notice") == 0);
     is_action = (ctcp_type && (weechat_strcasecmp (ctcp_type, "action") == 0));
 
@@ -129,6 +140,9 @@ irc_input_user_message_display (struct t_irc_server *server,
     if (ptr_channel && (ptr_channel->type == IRC_CHANNEL_TYPE_CHANNEL))
         ptr_nick = irc_nick_search (server, ptr_channel, server->nick);
 
+    ctxt.nick = (ptr_nick) ? ptr_nick->name : server->nick;
+    ctxt.nick_is_me = (irc_server_strcasecmp (server, ctxt.nick, server->nick) == 0);
+
     if (is_action)
     {
         snprintf (str_tags, sizeof (str_tags),
@@ -151,8 +165,7 @@ irc_input_user_message_display (struct t_irc_server *server,
                       "%sself_msg,notify_none,no_highlight,prefix_nick_%s",
                       (ctcp_type) ? "irc_ctcp," : "",
                       (str_color) ? str_color : "default");
-            if (str_color)
-                free (str_color);
+            free (str_color);
         }
     }
 
@@ -162,16 +175,11 @@ irc_input_user_message_display (struct t_irc_server *server,
     {
         if (display_target)
         {
-            weechat_printf_date_tags (
+            weechat_printf_datetime_tags (
                 ptr_buffer,
                 date,
-                irc_protocol_tags (
-                    server,
-                    command,
-                    NULL,
-                    str_tags,
-                    (ptr_nick) ? ptr_nick->name : server->nick,
-                    address),
+                date_usec,
+                irc_protocol_tags (&ctxt, str_tags),
                 "%s%s -> %s%s%s: %s%s%s%s%s%s",
                 weechat_prefix ("network"),
                 /* TRANSLATORS: "Action" is an IRC CTCP "ACTION" sent with /me or /action */
@@ -189,16 +197,11 @@ irc_input_user_message_display (struct t_irc_server *server,
         }
         else
         {
-            weechat_printf_date_tags (
+            weechat_printf_datetime_tags (
                 ptr_buffer,
                 date,
-                irc_protocol_tags (
-                    server,
-                    command,
-                    NULL,
-                    str_tags,
-                    (ptr_nick) ? ptr_nick->name : server->nick,
-                    address),
+                date_usec,
+                irc_protocol_tags (&ctxt, str_tags),
                 "%s%s%s%s%s%s%s",
                 weechat_prefix ("action"),
                 irc_nick_mode_for_display (server, ptr_nick, 0),
@@ -211,16 +214,11 @@ irc_input_user_message_display (struct t_irc_server *server,
     }
     else if (ctcp_type)
     {
-        weechat_printf_date_tags (
+        weechat_printf_datetime_tags (
             ptr_buffer,
             date,
-            irc_protocol_tags (
-                server,
-                command,
-                NULL,
-                str_tags,
-                (ptr_nick) ? ptr_nick->name : server->nick,
-                address),
+            date_usec,
+            irc_protocol_tags (&ctxt, str_tags),
             _("%sCTCP query to %s%s%s: %s%s%s%s%s"),
             weechat_prefix ("network"),
             (is_channel) ?
@@ -235,16 +233,11 @@ irc_input_user_message_display (struct t_irc_server *server,
     }
     else if (display_target)
     {
-        weechat_printf_date_tags (
+        weechat_printf_datetime_tags (
             ptr_buffer,
             date,
-            irc_protocol_tags (
-                server,
-                command,
-                NULL,
-                str_tags,
-                (ptr_nick) ? ptr_nick->name : server->nick,
-                address),
+            date_usec,
+            irc_protocol_tags (&ctxt, str_tags),
             "%s%s%s%s%s(%s%s%s%s)%s -> %s%s%s: %s",
             weechat_prefix ("network"),
             (is_notice) ? IRC_COLOR_NOTICE : "",
@@ -267,16 +260,11 @@ irc_input_user_message_display (struct t_irc_server *server,
     }
     else
     {
-        weechat_printf_date_tags (
+        weechat_printf_datetime_tags (
             ptr_buffer,
             date,
-            irc_protocol_tags (
-                server,
-                command,
-                NULL,
-                str_tags,
-                (ptr_nick) ? ptr_nick->name : server->nick,
-                address),
+            date_usec,
+            irc_protocol_tags (&ctxt, str_tags),
             "%s%s",
             irc_nick_as_prefix (
                 server,
@@ -286,10 +274,8 @@ irc_input_user_message_display (struct t_irc_server *server,
             (ptr_text) ? ptr_text : "");
     }
 
-    if (text2)
-        free (text2);
-    if (text_decoded)
-        free (text_decoded);
+    free (text2);
+    free (text_decoded);
 }
 
 /*
@@ -337,6 +323,8 @@ irc_input_send_user_message (struct t_gui_buffer *buffer, int flags,
                 irc_input_user_message_display (
                     ptr_server,
                     0,  /* date */
+                    0,  /* date_usec */
+                    NULL,  /* tags */
                     ptr_channel->name,
                     NULL,  /* address */
                     "privmsg",
@@ -415,8 +403,7 @@ irc_input_data (struct t_gui_buffer *buffer, const char *input_data, int flags,
                 free (msg);
             }
 
-            if (data_with_colors)
-                free (data_with_colors);
+            free (data_with_colors);
         }
         else
         {
@@ -589,8 +576,7 @@ irc_input_send_cb (const void *pointer, void *data,
                 weechat_command (
                     ptr_buffer,
                     (data_with_colors) ? data_with_colors : ptr_message);
-                if (data_with_colors)
-                    free (data_with_colors);
+                free (data_with_colors);
             }
 
             /* reset tags to use by default */
@@ -598,14 +584,10 @@ irc_input_send_cb (const void *pointer, void *data,
         }
     }
 
-    if (server)
-        free (server);
-    if (channel)
-        free (channel);
-    if (options)
-        free (options);
-    if (tags)
-        free (tags);
+    free (server);
+    free (channel);
+    free (options);
+    free (tags);
 
     return WEECHAT_RC_OK;
 }
