@@ -42,6 +42,7 @@
 struct t_irc_server *irc_upgrade_current_server = NULL;
 struct t_irc_channel *irc_upgrade_current_channel = NULL;
 struct t_irc_modelist *irc_upgrade_current_modelist = NULL;
+int irc_upgrading = 0;
 
 
 /*
@@ -499,16 +500,14 @@ irc_upgrade_read_cb (const void *pointer, void *data,
                     str = weechat_infolist_string (infolist, "prefix_modes");
                     if (str)
                     {
-                        if (irc_upgrade_current_server->prefix_modes)
-                            free (irc_upgrade_current_server->prefix_modes);
+                        free (irc_upgrade_current_server->prefix_modes);
                         irc_upgrade_current_server->prefix_modes = strdup (str);
                     }
                     /* "prefix_chars" is new in WeeChat 0.3.4 */
                     str = weechat_infolist_string (infolist, "prefix_chars");
                     if (str)
                     {
-                        if (irc_upgrade_current_server->prefix_chars)
-                            free (irc_upgrade_current_server->prefix_chars);
+                        free (irc_upgrade_current_server->prefix_chars);
                         irc_upgrade_current_server->prefix_chars = strdup (str);
                     }
                     /* "msg_max_length" is new in WeeChat 4.0.0 */
@@ -649,6 +648,7 @@ irc_upgrade_read_cb (const void *pointer, void *data,
                     irc_upgrade_current_server->reconnect_delay = weechat_infolist_integer (infolist, "reconnect_delay");
                     irc_upgrade_current_server->reconnect_start = weechat_infolist_time (infolist, "reconnect_start");
                     irc_upgrade_current_server->command_time = weechat_infolist_time (infolist, "command_time");
+                    irc_upgrade_current_server->autojoin_time = weechat_infolist_time (infolist, "autojoin_time");
                     irc_upgrade_current_server->autojoin_done = weechat_infolist_integer (infolist, "autojoin_done");
                     irc_upgrade_current_server->disable_autojoin = weechat_infolist_integer (infolist, "disable_autojoin");
                     irc_upgrade_current_server->is_away = weechat_infolist_integer (infolist, "is_away");
@@ -794,14 +794,15 @@ irc_upgrade_read_cb (const void *pointer, void *data,
             case IRC_UPGRADE_TYPE_NICK:
                 if (irc_upgrade_current_server && irc_upgrade_current_channel)
                 {
-                    ptr_nick = irc_nick_new (irc_upgrade_current_server,
-                                             irc_upgrade_current_channel,
-                                             weechat_infolist_string (infolist, "name"),
-                                             weechat_infolist_string (infolist, "host"),
-                                             weechat_infolist_string (infolist, "prefixes"),
-                                             weechat_infolist_integer (infolist, "away"),
-                                             weechat_infolist_string (infolist, "account"),
-                                             weechat_infolist_string (infolist, "realname"));
+                    ptr_nick = irc_nick_new_in_channel (
+                        irc_upgrade_current_server,
+                        irc_upgrade_current_channel,
+                        weechat_infolist_string (infolist, "name"),
+                        weechat_infolist_string (infolist, "host"),
+                        weechat_infolist_string (infolist, "prefixes"),
+                        weechat_infolist_integer (infolist, "away"),
+                        weechat_infolist_string (infolist, "account"),
+                        weechat_infolist_string (infolist, "realname"));
                     if (ptr_nick)
                     {
                         /*
@@ -982,6 +983,35 @@ irc_upgrade_read_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Set buffer properties on IRC buffers after upgrade:
+ *   - "input_prompt" (introduced in WeeChat 4.3.0)
+ *   - "modes" (introduced in WeeChat 4.3.0)
+ */
+
+void
+irc_upgrade_set_buffer_properties ()
+{
+    struct t_irc_server *ptr_server;
+    struct t_irc_channel *ptr_channel;
+
+    for (ptr_server = irc_servers; ptr_server;
+         ptr_server = ptr_server->next_server)
+    {
+        /* set input prompt on server and all channels */
+        if (ptr_server->buffer)
+            irc_server_set_buffer_input_prompt (ptr_server);
+
+        /* set modes on all channels */
+        for (ptr_channel = ptr_server->channels; ptr_channel;
+             ptr_channel = ptr_channel->next_channel)
+        {
+            if (ptr_channel->buffer)
+                irc_channel_set_buffer_modes (ptr_server, ptr_channel);
+        }
+    }
+}
+
 /*
  * Loads irc upgrade file.
  *
@@ -1004,7 +1034,9 @@ irc_upgrade_load ()
     if (!upgrade_file)
         return 0;
 
+    irc_upgrading = 1;
     rc = weechat_upgrade_read (upgrade_file);
+    irc_upgrading = 0;
 
     weechat_upgrade_close (upgrade_file);
 
@@ -1016,5 +1048,7 @@ irc_upgrade_load ()
             (ptr_filter && ptr_filter[0]) ? ptr_filter : "*");
     }
 
+    irc_upgrade_set_buffer_properties ();
+
     return rc;
 }
