@@ -2,7 +2,7 @@
  * weechat-js.cpp - javascript plugin for WeeChat
  *
  * Copyright (C) 2013 Koka El Kiwi <kokakiwi@kokakiwi.net>
- * Copyright (C) 2015-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2015-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -452,8 +452,7 @@ weechat_js_unload (struct t_plugin_script *script)
     {
         rc = (int *)weechat_js_exec (script, WEECHAT_SCRIPT_EXEC_INT,
                                      script->shutdown_func, NULL, NULL);
-        if (rc)
-            free (rc);
+        free (rc);
     }
 
     filename = strdup (script->filename);
@@ -473,8 +472,7 @@ weechat_js_unload (struct t_plugin_script *script)
 
     (void) weechat_hook_signal_send ("javascript_script_unloaded",
                                      WEECHAT_HOOK_SIGNAL_STRING, filename);
-    if (filename)
-        free (filename);
+    free (filename);
 }
 
 /*
@@ -584,7 +582,7 @@ weechat_js_command_cb (const void *pointer, void *data,
                        int argc, char **argv, char **argv_eol)
 {
     char *ptr_name, *ptr_code, *path_script;
-    int i, send_to_buffer_as_input, exec_commands;
+    int i, send_to_buffer_as_input, exec_commands, old_js_quiet;
 
     /* make C++ compiler happy */
     (void) pointer;
@@ -643,6 +641,7 @@ weechat_js_command_cb (const void *pointer, void *data,
                  || (weechat_strcmp (argv[1], "reload") == 0)
                  || (weechat_strcmp (argv[1], "unload") == 0))
         {
+            old_js_quiet = js_quiet;
             ptr_name = argv_eol[2];
             if (strncmp (ptr_name, "-q ", 3) == 0)
             {
@@ -660,8 +659,7 @@ weechat_js_command_cb (const void *pointer, void *data,
                                                          ptr_name, 1);
                 weechat_js_load ((path_script) ? path_script : ptr_name,
                                  NULL);
-                if (path_script)
-                    free (path_script);
+                free (path_script);
             }
             else if (weechat_strcmp (argv[1], "reload") == 0)
             {
@@ -673,7 +671,7 @@ weechat_js_command_cb (const void *pointer, void *data,
                 /* unload javascript script */
                 weechat_js_unload_name (ptr_name);
             }
-            js_quiet = 0;
+            js_quiet = old_js_quiet;
         }
         else if (weechat_strcmp (argv[1], "eval") == 0)
         {
@@ -923,6 +921,7 @@ EXPORT int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
     char str_interpreter[64];
+    int old_js_quiet;
 
     /* make C compiler happy */
     (void) argc;
@@ -930,6 +929,11 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     weechat_js_plugin = plugin;
 
+    js_quiet = 0;
+    js_eval_mode = 0;
+    js_eval_send_input = 0;
+    js_eval_exec_commands = 0;
+
     /* set interpreter name and version */
     snprintf (str_interpreter, sizeof (str_interpreter),
               "%s (v8)", plugin->name);
@@ -951,11 +955,13 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     js_data.callback_signal_debug_dump = &weechat_js_signal_debug_dump_cb;
     js_data.callback_signal_script_action = &weechat_js_signal_script_action_cb;
     js_data.callback_load_file = &weechat_js_load_cb;
+    js_data.init_before_autoload = NULL;
     js_data.unload_all = &weechat_js_unload_all;
 
+    old_js_quiet = js_quiet;
     js_quiet = 1;
     plugin_script_init (plugin, &js_data);
-    js_quiet = 0;
+    js_quiet = old_js_quiet;
 
     plugin_script_display_short_list (weechat_js_plugin, js_scripts);
 
@@ -969,6 +975,9 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 EXPORT int
 weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
+    int old_js_quiet;
+
+    old_js_quiet = js_quiet;
     js_quiet = 1;
     if (js_script_eval)
     {
@@ -976,15 +985,24 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
         js_script_eval = NULL;
     }
     plugin_script_end (plugin, &js_data);
-    js_quiet = 0;
+    js_quiet = old_js_quiet;
 
     /* free some data */
     if (js_action_install_list)
+    {
         free (js_action_install_list);
+        js_action_install_list = NULL;
+    }
     if (js_action_remove_list)
+    {
         free (js_action_remove_list);
+        js_action_remove_list = NULL;
+    }
     if (js_action_autoload_list)
+    {
         free (js_action_autoload_list);
+        js_action_autoload_list = NULL;
+    }
     /* weechat_string_dyn_free (js_buffer_output, 1); */
 
     return WEECHAT_RC_OK;
