@@ -1,7 +1,7 @@
 /*
  * plugin-api-info.c - extra info functions for plugin API
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -30,17 +30,17 @@
 #include <time.h>
 
 #include "../core/weechat.h"
-#include "../core/wee-config.h"
-#include "../core/wee-crypto.h"
-#include "../core/wee-hashtable.h"
-#include "../core/wee-hook.h"
-#include "../core/wee-infolist.h"
-#include "../core/wee-proxy.h"
-#include "../core/wee-secure.h"
-#include "../core/wee-string.h"
-#include "../core/wee-url.h"
-#include "../core/wee-util.h"
-#include "../core/wee-version.h"
+#include "../core/core-config.h"
+#include "../core/core-crypto.h"
+#include "../core/core-hashtable.h"
+#include "../core/core-hook.h"
+#include "../core/core-infolist.h"
+#include "../core/core-proxy.h"
+#include "../core/core-secure.h"
+#include "../core/core-string.h"
+#include "../core/core-url.h"
+#include "../core/core-util.h"
+#include "../core/core-version.h"
 #include "../gui/gui-bar.h"
 #include "../gui/gui-bar-item.h"
 #include "../gui/gui-bar-window.h"
@@ -54,6 +54,7 @@
 #include "../gui/gui-key.h"
 #include "../gui/gui-layout.h"
 #include "../gui/gui-line.h"
+#include "../gui/gui-mouse.h"
 #include "../gui/gui-nick.h"
 #include "../gui/gui-nicklist.h"
 #include "../gui/gui-window.h"
@@ -135,7 +136,7 @@ plugin_api_info_date_cb (const void *pointer, void *data,
                          const char *info_name,
                          const char *arguments)
 {
-    const char *version;
+    const char *compilation_date;
 
     /* make C compiler happy */
     (void) pointer;
@@ -143,8 +144,8 @@ plugin_api_info_date_cb (const void *pointer, void *data,
     (void) info_name;
     (void) arguments;
 
-    version = version_get_compilation_date_time ();
-    return (version) ? strdup (version) : NULL;
+    compilation_date = version_get_compilation_date_time ();
+    return (compilation_date) ? strdup (compilation_date) : NULL;
 }
 
 /*
@@ -197,7 +198,7 @@ plugin_api_info_absolute_path (const char *directory)
 {
     char absolute_path[PATH_MAX];
 
-    if (!realpath (directory, absolute_path))
+    if (!directory || !realpath (directory, absolute_path))
         return NULL;
     return strdup ((absolute_path[0]) ? absolute_path : directory);
 }
@@ -238,6 +239,24 @@ plugin_api_info_weechat_data_dir_cb (const void *pointer, void *data,
     return plugin_api_info_absolute_path (weechat_data_dir);
 }
 
+/*
+ * Returns WeeChat info "weechat_state_dir".
+ */
+
+char *
+plugin_api_info_weechat_state_dir_cb (const void *pointer, void *data,
+                                      const char *info_name,
+                                      const char *arguments)
+{
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+    (void) arguments;
+
+    return plugin_api_info_absolute_path (weechat_state_dir);
+}
+
 /*
  * Returns WeeChat info "weechat_cache_dir".
  */
@@ -624,6 +643,27 @@ plugin_api_info_cursor_mode_cb (const void *pointer, void *data,
     return strdup (value);
 }
 
+/*
+ * Returns WeeChat info "mouse".
+ */
+
+char *
+plugin_api_info_mouse_cb (const void *pointer, void *data,
+                          const char *info_name,
+                          const char *arguments)
+{
+    char value[32];
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+    (void) arguments;
+
+    snprintf (value, sizeof (value), "%d", gui_mouse_enabled);
+    return strdup (value);
+}
+
 /*
  * Returns WeeChat info "term_width".
  */
@@ -818,8 +858,7 @@ plugin_api_info_nick_color_cb (const void *pointer, void *data,
         -1,
         (num_items >= 2) ? items[1] : NULL);
 
-    if (items)
-        string_free_split (items);
+    string_free_split (items);
 
     return result;
 }
@@ -848,8 +887,7 @@ plugin_api_info_nick_color_name_cb (const void *pointer, void *data,
         -1,
         (num_items >= 2) ? items[1] : NULL);
 
-    if (items)
-        string_free_split (items);
+    string_free_split (items);
 
     return result;
 }
@@ -877,6 +915,7 @@ plugin_api_info_nick_color_ignore_case_cb (const void *pointer, void *data,
     case_range = -1;
     if (num_items >= 2)
     {
+        error = NULL;
         number = strtol (items[1], &error, 10);
         if (error && !error[0])
             case_range = (int)number;
@@ -887,8 +926,7 @@ plugin_api_info_nick_color_ignore_case_cb (const void *pointer, void *data,
         case_range,
         (num_items >= 3) ? items[2] : NULL);
 
-    if (items)
-        string_free_split (items);
+    string_free_split (items);
 
     return result;
 }
@@ -916,6 +954,7 @@ plugin_api_info_nick_color_name_ignore_case_cb (const void *pointer, void *data,
     case_range = -1;
     if (num_items >= 2)
     {
+        error = NULL;
         number = strtol (items[1], &error, 10);
         if (error && !error[0])
             case_range = (int)number;
@@ -926,8 +965,7 @@ plugin_api_info_nick_color_name_ignore_case_cb (const void *pointer, void *data,
         case_range,
         (num_items >= 3) ? items[2] : NULL);
 
-    if (items)
-        string_free_split (items);
+    string_free_split (items);
 
     return result;
 }
@@ -1077,10 +1115,8 @@ plugin_api_info_totp_generate_cb (const void *pointer, void *data,
     return totp;
 
 error:
-    if (argv)
-        string_free_split (argv);
-    if (totp)
-        free (totp);
+    string_free_split (argv);
+    free (totp);
     return NULL;
 }
 
@@ -1150,11 +1186,77 @@ plugin_api_info_totp_validate_cb (const void *pointer, void *data,
     return strdup (value);
 
 error:
-    if (argv)
-        string_free_split (argv);
+    string_free_split (argv);
     return NULL;
 }
 
+/*
+ * Returns WeeChat info "plugin_loaded".
+ */
+
+char *
+plugin_api_info_plugin_loaded_cb (const void *pointer, void *data,
+                                  const char *info_name,
+                                  const char *arguments)
+{
+    struct t_weechat_plugin *ptr_plugin;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+
+    if (!arguments || !arguments[0])
+        return NULL;
+
+    for (ptr_plugin = weechat_plugins; ptr_plugin;
+         ptr_plugin = ptr_plugin->next_plugin)
+    {
+        if (strcmp (ptr_plugin->name, arguments) == 0)
+        {
+            /* plugin loaded */
+            return strdup ("1");
+        }
+    }
+
+    /* plugin not loaded */
+    return NULL;
+}
+
+/*
+ * Returns WeeChat info "window".
+ */
+
+char *
+plugin_api_info_window_cb (const void *pointer, void *data,
+                           const char *info_name,
+                           const char *arguments)
+{
+    struct t_gui_window *ptr_window;
+    long number;
+    char *error, value[64];
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+
+    if (!arguments || !arguments[0])
+        return NULL;
+
+    error = NULL;
+    number = (int)strtol (arguments, &error, 10);
+    if (!error || error[0])
+        return NULL;
+
+    ptr_window = gui_window_search_by_number (number);
+    if (!ptr_window)
+        return NULL;
+
+    snprintf (value, sizeof (value), "0x%lx", (unsigned long)ptr_window);
+    return strdup (value);
+}
+
 /*
  * Returns secured data hashtable.
  */
@@ -1643,24 +1745,24 @@ plugin_api_infolist_key_cb (const void *pointer, void *data,
     (void) infolist_name;
     (void) obj_pointer;
 
-    ptr_infolist = infolist_new (NULL);
-    if (!ptr_infolist)
-        return NULL;
-
     if (arguments && arguments[0])
         context = gui_key_search_context (arguments);
     else
         context = GUI_KEY_CONTEXT_DEFAULT;
-    if (context >= 0)
+    if (context < 0)
+        return NULL;
+
+    ptr_infolist = infolist_new (NULL);
+    if (!ptr_infolist)
+        return NULL;
+
+    for (ptr_key = gui_keys[context]; ptr_key;
+         ptr_key = ptr_key->next_key)
     {
-        for (ptr_key = gui_keys[context]; ptr_key;
-             ptr_key = ptr_key->next_key)
+        if (!gui_key_add_to_infolist (ptr_infolist, ptr_key))
         {
-            if (!gui_key_add_to_infolist (ptr_infolist, ptr_key))
-            {
-                infolist_free (ptr_infolist);
-                return NULL;
-            }
+            infolist_free (ptr_infolist);
+            return NULL;
         }
     }
     return ptr_infolist;
@@ -2052,7 +2154,8 @@ plugin_api_info_init ()
                N_("WeeChat directory "
                   "(*deprecated* since version 3.2, replaced by "
                   "\"weechat_config_dir\", \"weechat_data_dir\", "
-                  "\"weechat_cache_dir\" and \"weechat_runtime_dir\")"),
+                  "\"weechat_state_dir\" , \"weechat_cache_dir\" "
+                  "and \"weechat_runtime_dir\")"),
                NULL, &plugin_api_info_weechat_data_dir_cb, NULL, NULL);
     hook_info (NULL, "weechat_config_dir",
                N_("WeeChat config directory"),
@@ -2060,6 +2163,9 @@ plugin_api_info_init ()
     hook_info (NULL, "weechat_data_dir",
                N_("WeeChat data directory"),
                NULL, &plugin_api_info_weechat_data_dir_cb, NULL, NULL);
+    hook_info (NULL, "weechat_state_dir",
+               N_("WeeChat state directory"),
+               NULL, &plugin_api_info_weechat_state_dir_cb, NULL, NULL);
     hook_info (NULL, "weechat_cache_dir",
                N_("WeeChat cache directory"),
                NULL, &plugin_api_info_weechat_cache_dir_cb, NULL, NULL);
@@ -2123,6 +2229,9 @@ plugin_api_info_init ()
     hook_info (NULL, "cursor_mode",
                N_("1 if cursor mode is enabled"),
                NULL, &plugin_api_info_cursor_mode_cb, NULL, NULL);
+    hook_info (NULL, "mouse",
+               N_("1 if mouse is enabled"),
+               NULL, &plugin_api_info_mouse_cb, NULL, NULL);
     hook_info (NULL, "term_width",
                N_("width of terminal"),
                NULL, &plugin_api_info_term_width_cb, NULL, NULL);
@@ -2207,6 +2316,14 @@ plugin_api_info_init ()
                   "timestamp (optional, current time by default), number of "
                   "passwords before/after to test (optional, 0 by default)"),
                &plugin_api_info_totp_validate_cb, NULL, NULL);
+    hook_info (NULL, "plugin_loaded",
+               N_("1 if plugin is loaded"),
+               N_("plugin name"),
+               &plugin_api_info_plugin_loaded_cb, NULL, NULL);
+    hook_info (NULL, "window",
+               N_("window pointer"),
+               N_("window number"),
+               &plugin_api_info_window_cb, NULL, NULL);
 
     /* WeeChat core info_hashtable hooks */
     hook_info_hashtable (NULL,
