@@ -1,7 +1,7 @@
 /*
  * plugin-script.c - common functions used by script plugins
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -36,6 +36,54 @@
 #include "plugin-script-config.h"
 
 
+struct t_weechat_script_constant weechat_script_constants[] = {
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_RC_OK),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_RC_OK_EAT),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_RC_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_READ_OK),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_READ_MEMORY_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_READ_FILE_NOT_FOUND),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_WRITE_OK),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_WRITE_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_WRITE_MEMORY_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_OPTION_SET_OK_CHANGED),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_OPTION_SET_OK_SAME_VALUE),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_OPTION_SET_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_OPTION_SET_OPTION_NOT_FOUND),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_OPTION_UNSET_OK_NO_RESET),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_OPTION_UNSET_OK_RESET),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_OPTION_UNSET_OK_REMOVED),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_CONFIG_OPTION_UNSET_ERROR),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_LIST_POS_SORT),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_LIST_POS_BEGINNING),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_LIST_POS_END),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_HOTLIST_LOW),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_HOTLIST_MESSAGE),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_HOTLIST_PRIVATE),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_HOTLIST_HIGHLIGHT),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_PROCESS_RUNNING),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_PROCESS_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_IPV6_DISABLE),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_IPV6_AUTO),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_IPV6_FORCE),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_OK),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_ADDRESS_NOT_FOUND),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_IP_ADDRESS_NOT_FOUND),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_CONNECTION_REFUSED),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_PROXY_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_LOCAL_HOSTNAME_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_GNUTLS_HANDSHAKE_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_MEMORY_ERROR),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_TIMEOUT),
+    WEECHAT_SCRIPT_CONST_INT(WEECHAT_HOOK_CONNECT_SOCKET_ERROR),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_HOOK_SIGNAL_STRING),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_HOOK_SIGNAL_INT),
+    WEECHAT_SCRIPT_CONST_STR(WEECHAT_HOOK_SIGNAL_POINTER),
+    { NULL, 0, NULL },
+};
+
+
 /*
  * Displays name and version of interpreter used.
  */
@@ -201,31 +249,31 @@ plugin_script_init (struct t_weechat_plugin *weechat_plugin,
            " || reload|unload [-q] [<name>]"
            " || eval [-o|-oc] <code>"
            " || version"),
-        N_("    list: list loaded scripts\n"
-           "listfull: list loaded scripts (verbose)\n"
-           "    load: load a script\n"
-           "autoload: load all scripts in \"autoload\" directory\n"
-           "  reload: reload a script (if no name given, unload all scripts, "
-           "then load all scripts in \"autoload\" directory)\n"
-           "  unload: unload a script (if no name given, unload all scripts)\n"
-           "filename: script (file) to load\n"
-           "      -q: quiet mode: do not display messages\n"
-           "    name: a script name (name used in call to \"register\" "
-           "function)\n"
-           "    eval: evaluate source code and display result on current "
-           "buffer\n"
-           "      -o: send evaluation result to the buffer without executing "
-           "commands\n"
-           "     -oc: send evaluation result to the buffer and execute "
-           "commands\n"
-           "    code: source code to evaluate\n"
-           " version: display the version of interpreter used\n"
-           "\n"
-           "Without argument, this command lists all loaded scripts."),
+        WEECHAT_CMD_ARGS_DESC(
+            N_("raw[list]: list loaded scripts"),
+            N_("raw[listfull]: list loaded scripts (verbose)"),
+            N_("raw[load]: load a script"),
+            N_("raw[autoload]: load all scripts in \"autoload\" directory"),
+            N_("raw[reload]: reload a script (if no name given, unload all scripts, "
+               "then load all scripts in \"autoload\" directory)"),
+            N_("raw[unload]: unload a script (if no name given, unload all scripts)"),
+            N_("filename: script (file) to load"),
+            N_("raw[-q]: quiet mode: do not display messages"),
+            N_("name: a script name (name used in call to \"register\" "
+               "function)"),
+            N_("raw[eval]: evaluate source code and display result on current "
+               "buffer"),
+            N_("raw[-o]: send evaluation result to the buffer without executing "
+               "commands"),
+            N_("raw[-oc]: send evaluation result to the buffer and execute "
+               "commands"),
+            N_("code: source code to evaluate"),
+            N_("raw[version]: display the version of interpreter used"),
+            "",
+            N_("Without argument, this command lists all loaded scripts.")),
         completion,
         plugin_data->callback_command, NULL, NULL);
-    if (completion)
-        free (completion);
+    free (completion);
 
     /* add completion, hdata and infolist */
     snprintf (string, sizeof (string), "%s_script", weechat_plugin->name);
@@ -272,13 +320,19 @@ plugin_script_init (struct t_weechat_plugin *weechat_plugin,
                        &plugin_script_info_version_cb,
                        weechat_plugin, NULL);
 
+    /*
+     * call function "init_before_autoload"
+     * (called even if no scripts are auto-loaded)
+     */
+    if (plugin_data->init_before_autoload)
+        (void)(plugin_data->init_before_autoload) ();
+
     /* check if auto-load of scripts is enabled */
     info_auto_load_scripts = weechat_info_get ("auto_load_scripts", NULL);
     auto_load_scripts = (info_auto_load_scripts
                          && (strcmp (info_auto_load_scripts, "1") == 0)) ?
         1 : 0;
-    if (info_auto_load_scripts)
-        free (info_auto_load_scripts);
+    free (info_auto_load_scripts);
 
     /* autoload scripts */
     if (auto_load_scripts)
@@ -351,7 +405,7 @@ plugin_script_str2ptr (struct t_weechat_plugin *weechat_plugin,
                        const char *script_name, const char *function_name,
                        const char *str_pointer)
 {
-    unsigned long value;
+    void *pointer;
     int rc;
     struct t_gui_buffer *ptr_buffer;
 
@@ -361,9 +415,9 @@ plugin_script_str2ptr (struct t_weechat_plugin *weechat_plugin,
     if ((str_pointer[0] != '0') || (str_pointer[1] != 'x'))
         goto invalid;
 
-    rc = sscanf (str_pointer + 2, "%lx", &value);
+    rc = sscanf (str_pointer, "%p", &pointer);
     if ((rc != EOF) && (rc >= 1))
-        return (void *)value;
+        return pointer;
 
 invalid:
     if ((weechat_plugin->debug >= 1) && script_name && function_name)
@@ -1021,22 +1075,14 @@ plugin_script_remove_configs (struct t_weechat_plugin *weechat_plugin,
 void
 plugin_script_free (struct t_plugin_script *script)
 {
-    if (script->filename)
-        free (script->filename);
-    if (script->name)
-        free (script->name);
-    if (script->author)
-        free (script->author);
-    if (script->version)
-        free (script->version);
-    if (script->license)
-        free (script->license);
-    if (script->description)
-        free (script->description);
-    if (script->shutdown_func)
-        free (script->shutdown_func);
-    if (script->charset)
-        free (script->charset);
+    free (script->filename);
+    free (script->name);
+    free (script->author);
+    free (script->version);
+    free (script->license);
+    free (script->description);
+    free (script->shutdown_func);
+    free (script->charset);
 
     free (script);
 }
@@ -1053,13 +1099,19 @@ plugin_script_remove (struct t_weechat_plugin *weechat_plugin,
 {
     script->unloading = 1;
 
+    /* remove all hooks created by this script */
+    weechat_unhook_all (script->name);
+
     plugin_script_close_buffers (weechat_plugin, script);
 
     plugin_script_remove_bar_items (weechat_plugin, script);
 
     plugin_script_remove_configs (weechat_plugin, script);
 
-    /* remove all hooks created by this script */
+    /*
+     * remove again all hooks created by this script (just in case new hooks
+     * were created by the calls above)
+     */
     weechat_unhook_all (script->name);
 
     /* remove script from list */
@@ -1317,8 +1369,7 @@ plugin_script_action_install (struct t_weechat_plugin *weechat_plugin,
                                         free (symlink_path);
                                     }
                                     free (autoload_path);
-                                    if (dir_separator)
-                                        free (dir_separator);
+                                    free (dir_separator);
                                 }
                             }
 
@@ -1345,8 +1396,7 @@ plugin_script_action_install (struct t_weechat_plugin *weechat_plugin,
                         free (new_path);
                     }
                     free (base_name);
-                    if (weechat_data_dir)
-                        free (weechat_data_dir);
+                    free (weechat_data_dir);
 
                     /* send signal */
                     snprintf (str_signal, sizeof (str_signal),
@@ -1517,8 +1567,7 @@ plugin_script_action_autoload (struct t_weechat_plugin *weechat_plugin,
                                 (void) rc;
                                 free (symlink_path);
                             }
-                            if (dir_separator)
-                                free (dir_separator);
+                            free (dir_separator);
                         }
                         else
                         {
@@ -1527,8 +1576,7 @@ plugin_script_action_autoload (struct t_weechat_plugin *weechat_plugin,
                         free (autoload_path);
                     }
                     free (base_name);
-                    if (weechat_data_dir)
-                        free (weechat_data_dir);
+                    free (weechat_data_dir);
                 }
                 free (name);
             }
@@ -1796,6 +1844,7 @@ plugin_script_end (struct t_weechat_plugin *weechat_plugin,
     /* write config file (file: "<language>.conf") */
     weechat_config_write (*(plugin_data->config_file));
     weechat_config_free (*(plugin_data->config_file));
+    *(plugin_data->config_file) = NULL;
 }
 
 /*
@@ -1816,19 +1865,19 @@ plugin_script_print_log (struct t_weechat_plugin *weechat_plugin,
          ptr_script = ptr_script->next_script)
     {
         weechat_log_printf ("");
-        weechat_log_printf ("[script %s (addr:0x%lx)]",      ptr_script->name, ptr_script);
-        weechat_log_printf ("  filename. . . . . . : '%s'",  ptr_script->filename);
-        weechat_log_printf ("  interpreter . . . . : 0x%lx", ptr_script->interpreter);
-        weechat_log_printf ("  name. . . . . . . . : '%s'",  ptr_script->name);
-        weechat_log_printf ("  author. . . . . . . : '%s'",  ptr_script->author);
-        weechat_log_printf ("  version . . . . . . : '%s'",  ptr_script->version);
-        weechat_log_printf ("  license . . . . . . : '%s'",  ptr_script->license);
-        weechat_log_printf ("  description . . . . : '%s'",  ptr_script->description);
-        weechat_log_printf ("  shutdown_func . . . : '%s'",  ptr_script->shutdown_func);
-        weechat_log_printf ("  charset . . . . . . : '%s'",  ptr_script->charset);
-        weechat_log_printf ("  unloading . . . . . : %d",    ptr_script->unloading);
-        weechat_log_printf ("  prev_script . . . . : 0x%lx", ptr_script->prev_script);
-        weechat_log_printf ("  next_script . . . . : 0x%lx", ptr_script->next_script);
+        weechat_log_printf ("[script %s (addr:%p)]", ptr_script->name, ptr_script);
+        weechat_log_printf ("  filename. . . . . . : '%s'", ptr_script->filename);
+        weechat_log_printf ("  interpreter . . . . : %p", ptr_script->interpreter);
+        weechat_log_printf ("  name. . . . . . . . : '%s'", ptr_script->name);
+        weechat_log_printf ("  author. . . . . . . : '%s'", ptr_script->author);
+        weechat_log_printf ("  version . . . . . . : '%s'", ptr_script->version);
+        weechat_log_printf ("  license . . . . . . : '%s'", ptr_script->license);
+        weechat_log_printf ("  description . . . . : '%s'", ptr_script->description);
+        weechat_log_printf ("  shutdown_func . . . : '%s'", ptr_script->shutdown_func);
+        weechat_log_printf ("  charset . . . . . . : '%s'", ptr_script->charset);
+        weechat_log_printf ("  unloading . . . . . : %d", ptr_script->unloading);
+        weechat_log_printf ("  prev_script . . . . : %p", ptr_script->prev_script);
+        weechat_log_printf ("  next_script . . . . : %p", ptr_script->next_script);
     }
 
     weechat_log_printf ("");
