@@ -33,6 +33,7 @@
 #include "../../../weechat-plugin.h"
 #include "../../relay.h"
 #include "../../relay-auth.h"
+#include "../../relay-buffer.h"
 #include "../../relay-config.h"
 #include "../../relay-http.h"
 #include "../../relay-raw.h"
@@ -88,8 +89,8 @@ relay_remote_event_search_buffer (struct t_relay_remote *remote, long long id)
         ptr_id = weechat_buffer_get_string (ptr_buffer, "localvar_relay_remote_id");
         if (ptr_name
             && ptr_id
-            && (strcmp (ptr_name, remote->name) == 0)
-            && (strcmp (ptr_id, str_id) == 0))
+            && (weechat_strcmp (ptr_name, remote->name) == 0)
+            && (weechat_strcmp (ptr_id, str_id) == 0))
         {
             ptr_buffer_found = ptr_buffer;
             break;
@@ -129,20 +130,84 @@ relay_remote_event_get_buffer_id (struct t_gui_buffer *buffer)
 }
 
 /*
- * Callback for a line event.
+ * Builds a string with tags of a received line.
+ *
+ * Returns dynamically string built, NULL if error.
  */
 
-RELAY_REMOTE_EVENT_CALLBACK(line)
+char **
+relay_remote_build_string_tags (cJSON *json_tags, int line_id, int highlight)
 {
-    cJSON *json_obj, *json_tags, *json_tag;
-    const char *date, *prefix, *message, *ptr_tag;
+    cJSON *json_tag;
+    const char *ptr_tag;
+    char **tags, str_tag_id[512];
+    int tag_notify_highlight;
+
+    tags = weechat_string_dyn_alloc (256);
+    if (!tags)
+        return NULL;
+
+    tag_notify_highlight = 0;
+
+    if (json_tags && cJSON_IsArray (json_tags))
+    {
+        cJSON_ArrayForEach (json_tag, json_tags)
+        {
+            ptr_tag = cJSON_GetStringValue (json_tag);
+            if (ptr_tag)
+            {
+                if ((*tags)[0])
+                    weechat_string_dyn_concat (tags, ",", -1);
+                if (highlight && (strncmp (ptr_tag, "notify_", 7) == 0))
+                {
+                    weechat_string_dyn_concat (tags, "notify_highlight", -1);
+                    tag_notify_highlight = 1;
+                }
+                else
+                {
+                    weechat_string_dyn_concat (tags, ptr_tag, -1);
+                }
+            }
+        }
+    }
+
+    /*
+     * add "notify_highlight" if line has highlight but no highlight tag
+     * was present
+     */
+    if (highlight && !tag_notify_highlight)
+    {
+        if ((*tags)[0])
+            weechat_string_dyn_concat (tags, ",", -1);
+        weechat_string_dyn_concat (tags, "notify_highlight", -1);
+    }
+
+    /* add tag with remote line id */
+    snprintf (str_tag_id, sizeof (str_tag_id), "relay_remote_line_id_%d", line_id);
+    if ((*tags)[0])
+        weechat_string_dyn_concat (tags, ",", -1);
+    weechat_string_dyn_concat (tags, str_tag_id, -1);
+
+    return tags;
+}
+
+/*
+ * Adds a new line in a buffer.
+ */
+
+void
+relay_remote_event_line_add (struct t_relay_remote_event *event)
+{
+    cJSON *json_obj;
+    const char *date, *prefix, *message;
     char **tags;
-    int y, highlight, tag_notify_highlight;
+    int id, y, highlight;
     struct timeval tv_date;
 
-    if (!event->buffer || !event->json)
-        return WEECHAT_RC_OK;
+    if (!event || !event->buffer)
+        return;
 
+    JSON_GET_NUM(event->json, id, -1);
     JSON_GET_NUM(event->json, y, -1);
     JSON_GET_STR(event->json, date);
     JSON_GET_BOOL(event->json, highlight);
@@ -155,40 +220,8 @@ RELAY_REMOTE_EVENT_CALLBACK(line)
         tv_date.tv_usec = 0;
     }
 
-    tag_notify_highlight = 0;
-
-    tags = weechat_string_dyn_alloc (256);
-    if (tags)
-    {
-        json_tags = cJSON_GetObjectItem (event->json, "tags");
-        if (json_tags && cJSON_IsArray (json_tags))
-        {
-            cJSON_ArrayForEach (json_tag, json_tags)
-            {
-                ptr_tag = cJSON_GetStringValue (json_tag);
-                if (ptr_tag)
-                {
-                    if (*tags[0])
-                        weechat_string_dyn_concat (tags, ",", -1);
-                    if (highlight && (strncmp (ptr_tag, "notify_", 7) == 0))
-                    {
-                        weechat_string_dyn_concat (tags, "notify_highlight", -1);
-                        tag_notify_highlight = 1;
-                    }
-                    else
-                    {
-                        weechat_string_dyn_concat (tags, ptr_tag, -1);
-                    }
-                }
-            }
-        }
-        if (highlight && !tag_notify_highlight)
-        {
-            if (*tags[0])
-                weechat_string_dyn_concat (tags, ",", -1);
-            weechat_string_dyn_concat (tags, "notify_highlight", -1);
-        }
-    }
+    tags = relay_remote_build_string_tags (
+        cJSON_GetObjectItem (event->json, "tags"), id, highlight);
 
     if (y >= 0)
     {
@@ -219,6 +252,152 @@ RELAY_REMOTE_EVENT_CALLBACK(line)
     }
 
     weechat_string_dyn_free (tags, 1);
+}
+
+/*
+ * Searches a line by its id in relay remote buffer (by searching tag
+ * "relay_remote_id_123456", where 123456 is the line id).
+ *
+ * Returns pointer to line found, NULL if not found.
+ */
+
+struct t_gui_line *
+relay_remote_event_search_line_by_id (struct t_gui_buffer *buffer, int id)
+{
+    struct t_gui_lines *ptr_lines;
+    struct t_gui_line *ptr_line;
+    struct t_gui_line_data *ptr_line_data;
+    const char **tags, **ptr_tag;
+    char str_tag_id[512];
+
+    if (!buffer)
+        return NULL;
+
+    ptr_lines = weechat_hdata_pointer (relay_hdata_buffer, buffer, "own_lines");
+    if (!ptr_lines)
+        return NULL;
+
+    ptr_line = weechat_hdata_pointer (relay_hdata_lines, ptr_lines, "last_line");
+    if (!ptr_line)
+        return NULL;
+
+    snprintf (str_tag_id, sizeof (str_tag_id), "relay_remote_line_id_%d", id);
+
+    while (ptr_line)
+    {
+        ptr_line_data = weechat_hdata_pointer (relay_hdata_line, ptr_line, "data");
+        if (!ptr_line_data)
+            continue;
+        tags = weechat_hdata_pointer (relay_hdata_line_data, ptr_line_data, "tags_array");
+        if (tags)
+        {
+            for (ptr_tag = tags; *ptr_tag; ptr_tag++)
+            {
+                if (weechat_strcmp (*ptr_tag, str_tag_id) == 0)
+                    return ptr_line;
+            }
+        }
+        ptr_line = weechat_hdata_move (relay_hdata_line, ptr_line, -1);
+    }
+
+    /* line not found */
+    return NULL;
+}
+
+/*
+ * Updates a line in a buffer.
+ */
+
+void
+relay_remote_event_line_update (struct t_relay_remote_event *event)
+{
+    cJSON *json_obj;
+    struct t_gui_line *ptr_line;
+    struct t_gui_line_data *ptr_line_data;
+    const char *date, *date_printed, *prefix, *message;
+    char **tags, str_value[1024];
+    int id, highlight;
+    struct timeval tv_date, tv_date_printed;
+    struct t_hashtable *hashtable;
+
+    if (!event || !event->buffer)
+        return;
+
+    JSON_GET_NUM(event->json, id, -1);
+
+    ptr_line = relay_remote_event_search_line_by_id (event->buffer, id);
+    if (!ptr_line)
+        return;
+
+    ptr_line_data = weechat_hdata_pointer (relay_hdata_line, ptr_line, "data");
+    if (!ptr_line_data)
+        return;
+
+    JSON_GET_STR(event->json, date);
+    JSON_GET_STR(event->json, date_printed);
+    JSON_GET_BOOL(event->json, highlight);
+    JSON_GET_STR(event->json, prefix);
+    JSON_GET_STR(event->json, message);
+
+    if (!weechat_util_parse_time (date, &tv_date))
+    {
+        tv_date.tv_sec = 0;
+        tv_date.tv_usec = 0;
+    }
+
+    if (!weechat_util_parse_time (date_printed, &tv_date_printed))
+    {
+        tv_date_printed.tv_sec = 0;
+        tv_date_printed.tv_usec = 0;
+    }
+
+    hashtable = weechat_hashtable_new (32,
+                                       WEECHAT_HASHTABLE_STRING,
+                                       WEECHAT_HASHTABLE_STRING,
+                                       NULL,
+                                       NULL);
+    if (!hashtable)
+        return;
+
+    snprintf (str_value, sizeof (str_value), "%ld", tv_date.tv_sec);
+    weechat_hashtable_set (hashtable, "date", str_value);
+    snprintf (str_value, sizeof (str_value), "%ld", tv_date.tv_usec);
+    weechat_hashtable_set (hashtable, "date_usec", str_value);
+
+    snprintf (str_value, sizeof (str_value), "%ld", tv_date_printed.tv_sec);
+    weechat_hashtable_set (hashtable, "date_printed", str_value);
+    snprintf (str_value, sizeof (str_value), "%ld", tv_date_printed.tv_usec);
+    weechat_hashtable_set (hashtable, "date_usec_printed", str_value);
+
+    tags = relay_remote_build_string_tags (
+        cJSON_GetObjectItem (event->json, "tags"), id, highlight);
+    if (tags)
+    {
+        weechat_hashtable_set (hashtable, "tags_array", *tags);
+        weechat_string_dyn_free (tags, 1);
+    }
+
+    weechat_hashtable_set (hashtable, "prefix", prefix);
+    weechat_hashtable_set (hashtable, "message", message);
+
+    weechat_hdata_update (relay_hdata_line_data, ptr_line_data, hashtable);
+
+    weechat_hashtable_free (hashtable);
+}
+
+/*
+ * Callback for a line event.
+ */
+
+RELAY_REMOTE_EVENT_CALLBACK(line)
+{
+    if (!event || !event->buffer || !event->json)
+        return WEECHAT_RC_OK;
+
+    if (weechat_strcmp (event->name, "buffer_line_data_changed") == 0)
+        relay_remote_event_line_update (event);
+    else
+        relay_remote_event_line_add (event);
 
     return WEECHAT_RC_OK;
 }
@@ -367,7 +546,7 @@ RELAY_REMOTE_EVENT_CALLBACK(nick_group)
     cJSON *json_obj;
     long long id;
 
-    if (!event->buffer || !event->json)
+    if (!event || !event->buffer || !event->json)
         return WEECHAT_RC_OK;
 
     if (weechat_strcmp (event->name, "nicklist_group_removing") == 0)
@@ -397,7 +576,7 @@ RELAY_REMOTE_EVENT_CALLBACK(nick)
     cJSON *json_obj;
     long long id;
 
-    if (!event->buffer || !event->json)
+    if (!event || !event->buffer || !event->json)
         return WEECHAT_RC_OK;
 
     if (weechat_strcmp (event->name, "nicklist_nick_removing") == 0)
@@ -438,23 +617,21 @@ relay_remote_event_apply_props (void *data,
  * Callback for remote buffer input.
  */
 
-int
-relay_remote_event_buffer_input_cb (const void *pointer,
-                                    void *data,
-                                    struct t_gui_buffer *buffer,
-                                    const char *input_data)
+void
+relay_remote_event_buffer_input (struct t_gui_buffer *buffer,
+                                 const char *input_data)
 {
     struct t_relay_remote *ptr_remote;
     cJSON *json, *json_body;
     long long buffer_id;
 
-    /* make C compiler happy */
-    (void) pointer;
-    (void) data;
+    if (!buffer)
+        return;
 
-    ptr_remote = relay_remote_search (weechat_buffer_get_string (buffer, "localvar_relay_remote"));
+    ptr_remote = relay_remote_search (
+        weechat_buffer_get_string (buffer, "localvar_relay_remote"));
     if (!ptr_remote)
-        return WEECHAT_RC_OK;
+        return;
 
     json = NULL;
 
@@ -478,16 +655,24 @@ relay_remote_event_buffer_input_cb (const void *pointer,
                            cJSON_CreateString (input_data));
     cJSON_AddItemToObject (json, "body", json_body);
 
-    relay_remote_network_send_json (ptr_remote, json);
+    if (relay_remote_network_send_json (ptr_remote, json) <= 0)
+    {
+        weechat_printf (
+            NULL,
+            _("%sremote[%s]: unable to send data, disconnecting"),
+            weechat_prefix ("error"),
+            ptr_remote->name);
+        relay_remote_network_disconnect (ptr_remote);
+    }
 
     cJSON_Delete (json);
 
-    return WEECHAT_RC_OK;
+    return;
 
 error:
     if (json)
         cJSON_Delete (json);
-    return WEECHAT_RC_OK;
+    return;
 }
 
 /*
@@ -553,6 +738,91 @@ relay_remote_event_remove_localvar_cb (void *data,
     }
 }
 
+/*
+ * Initial sync of buffers: remove any remote buffer that is not received
+ * from the remote (case of buffers still present after a previous connection
+ * to the remove).
+ */
+
+void
+relay_remote_event_initial_sync_buffers (struct t_relay_remote_event *event)
+{
+    struct t_gui_buffer *ptr_buffer;
+    struct t_arraylist *remote_buffers;
+    struct t_hashtable *buffers_id;
+    const char *ptr_name, *ptr_id;
+    char str_id[64];
+    cJSON *json_buffer, *json_obj;
+    long long id;
+    int i, list_size;
+
+    if (!event || !event->remote)
+        return;
+
+    /* build a list of existing buffers for the remote */
+    remote_buffers = weechat_arraylist_new (32, 0, 0,
+                                            NULL, NULL, NULL, NULL);
+    if (!remote_buffers)
+    {
+        relay_remote_network_disconnect (event->remote);
+        return;
+    }
+    ptr_buffer = weechat_hdata_get_list (relay_hdata_buffer, "gui_buffers");
+    while (ptr_buffer)
+    {
+        ptr_name = weechat_buffer_get_string (ptr_buffer, "localvar_relay_remote");
+        if (ptr_name && (weechat_strcmp (ptr_name, event->remote->name) == 0))
+        {
+            weechat_arraylist_add (remote_buffers, ptr_buffer);
+        }
+        ptr_buffer = weechat_hdata_move (relay_hdata_buffer, ptr_buffer, 1);
+    }
+
+    /* build a list of remote buffers ids received in JSON */
+    buffers_id = weechat_hashtable_new (32,
+                                        WEECHAT_HASHTABLE_STRING,
+                                        WEECHAT_HASHTABLE_POINTER,
+                                        NULL,
+                                        NULL);
+    if (!buffers_id)
+    {
+        weechat_arraylist_free (remote_buffers);
+        relay_remote_network_disconnect (event->remote);
+        return;
+    }
+    if (event->json && cJSON_IsArray (event->json))
+    {
+        cJSON_ArrayForEach (json_buffer, event->json)
+        {
+            JSON_GET_NUM(json_buffer, id, -1);
+            snprintf (str_id, sizeof (str_id), "%lld", id);
+            weechat_hashtable_set (buffers_id, str_id, NULL);
+        }
+    }
+
+    /* close any remote buffer that was not received in JSON */
+    list_size = weechat_arraylist_size (remote_buffers);
+    for (i = 0; i < list_size; i++)
+    {
+        ptr_buffer = (struct t_gui_buffer *)weechat_arraylist_get (remote_buffers, i);
+        if (!weechat_hdata_check_pointer (
+                relay_hdata_buffer,
+                weechat_hdata_get_list (relay_hdata_buffer, "gui_buffers"),
+                ptr_buffer))
+        {
+            continue;
+        }
+        ptr_id = weechat_buffer_get_string (ptr_buffer, "localvar_relay_remote_id");
+        if (ptr_id && !weechat_hashtable_has_key (buffers_id, ptr_id))
+        {
+            weechat_buffer_close (ptr_buffer);
+        }
+    }
+
+    weechat_arraylist_free (remote_buffers);
+    weechat_hashtable_free (buffers_id);
+}
+
 /*
  * Callback for a buffer event or response to GET /api/buffers.
  */
@@ -570,10 +840,11 @@ RELAY_REMOTE_EVENT_CALLBACK(buffer)
     const char *ptr_key, *ptr_command;
     char *full_name, str_number[64], str_local_var[1024], *property;
     long long id;
-    int number, nicklist, nicklist_case_sensitive, nicklist_display_groups;
+    int number, hidden, nicklist, nicklist_case_sensitive;
+    int nicklist_display_groups, time_displayed;
     int apply_props, input_position, input_multiline;
 
-    if (!event->json)
+    if (!event || !event->json)
         return WEECHAT_RC_OK;
 
     JSON_GET_NUM(event->json, id, -1);
@@ -581,6 +852,7 @@ RELAY_REMOTE_EVENT_CALLBACK(buffer)
     JSON_GET_STR(event->json, short_name);
     JSON_GET_NUM(event->json, number, -1);
     JSON_GET_STR(event->json, type);
+    JSON_GET_BOOL(event->json, hidden);
     JSON_GET_STR(event->json, title);
     JSON_GET_STR(event->json, modes);
     JSON_GET_STR(event->json, input_prompt);
@@ -590,6 +862,7 @@ RELAY_REMOTE_EVENT_CALLBACK(buffer)
     JSON_GET_BOOL(event->json, nicklist);
     JSON_GET_BOOL(event->json, nicklist_case_sensitive);
     JSON_GET_BOOL(event->json, nicklist_display_groups);
+    JSON_GET_BOOL(event->json, time_displayed);
 
     buffer_props = weechat_hashtable_new (32,
                                           WEECHAT_HASHTABLE_STRING,
@@ -601,20 +874,11 @@ RELAY_REMOTE_EVENT_CALLBACK(buffer)
 
     /* buffer base properties */
     weechat_hashtable_set (buffer_props, "type", type);
+    weechat_hashtable_set (buffer_props, "hidden", (hidden) ? "1" : "0");
     weechat_hashtable_set (buffer_props, "short_name", short_name);
     weechat_hashtable_set (buffer_props, "title", title);
     weechat_hashtable_set (buffer_props, "modes", modes);
     weechat_hashtable_set (buffer_props, "input_prompt", input_prompt);
-    if (!event->buffer)
-    {
-        /*
-         * set input content and position only when the buffer is created;
-         * subsequent updates will be handled via the "input" callback
-         */
-        weechat_hashtable_set (buffer_props, "input", input);
-        snprintf (str_number, sizeof (str_number), "%d", input_position);
-        weechat_hashtable_set (buffer_props, "input_pos", str_number);
-    }
     weechat_hashtable_set (buffer_props, "input_multiline",
                            (input_multiline) ? "1" : "0");
     weechat_hashtable_set (buffer_props, "nicklist", (nicklist) ? "1" : "0");
@@ -622,6 +886,8 @@ RELAY_REMOTE_EVENT_CALLBACK(buffer)
                            (nicklist_case_sensitive) ? "1" : "0");
     weechat_hashtable_set (buffer_props, "nicklist_display_groups",
                            (nicklist_display_groups) ? "1" : "0");
+    weechat_hashtable_set (buffer_props, "time_for_each_line",
+                           (time_displayed) ? "1" : "0");
 
     /* extra properties for relay */
     weechat_hashtable_set (buffer_props,
@@ -637,6 +903,8 @@ RELAY_REMOTE_EVENT_CALLBACK(buffer)
     /* if buffer exists, set properties, otherwise create buffer */
     apply_props = 1;
     ptr_buffer = event->buffer;
+    if (!ptr_buffer)
+        ptr_buffer = relay_remote_event_search_buffer (event->remote, id);
     if (!ptr_buffer)
     {
         if (weechat_asprintf (&full_name, "remote.%s.%s", event->remote->name, name) >= 0)
@@ -644,9 +912,16 @@ RELAY_REMOTE_EVENT_CALLBACK(buffer)
             ptr_buffer = weechat_buffer_search ("relay", full_name);
             if (!ptr_buffer)
             {
+                /*
+                 * set input content and position only when the buffer is created;
+                 * subsequent updates will be handled via the "input" callback
+                 */
+                weechat_hashtable_set (buffer_props, "input", input);
+                snprintf (str_number, sizeof (str_number), "%d", input_position);
+                weechat_hashtable_set (buffer_props, "input_pos", str_number);
                 ptr_buffer = weechat_buffer_new_props (
                     full_name, buffer_props,
-                    &relay_remote_event_buffer_input_cb, NULL, NULL,
+                    &relay_buffer_input_cb, NULL, NULL,
                     NULL, NULL, NULL);
                 apply_props = 0;
             }
@@ -734,6 +1009,7 @@ RELAY_REMOTE_EVENT_CALLBACK(buffer)
     json_lines = cJSON_GetObjectItem (event->json, "lines");
     if (json_lines && cJSON_IsArray (json_lines))
     {
+        event_line.name = "buffer_line_added";
         event_line.remote = event->remote;
         event_line.buffer = ptr_buffer;
         cJSON_ArrayForEach (json_line, json_lines)
@@ -785,7 +1061,7 @@ RELAY_REMOTE_EVENT_CALLBACK(input)
     char str_pos[64];
     int input_position;
 
-    if (!event->buffer || !event->json)
+    if (!event || !event->buffer || !event->json)
         return WEECHAT_RC_OK;
 
     JSON_GET_STR(event->json, input_prompt);
@@ -800,6 +1076,32 @@ RELAY_REMOTE_EVENT_CALLBACK(input)
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Clears buffers lines/nicklist for a remote.
+ */
+
+void
+relay_remote_event_clear_buffers (struct t_relay_remote *remote)
+{
+    struct t_gui_buffer *ptr_buffer;
+    const char *ptr_name;
+
+    if (!remote)
+        return;
+
+    ptr_buffer = weechat_hdata_get_list (relay_hdata_buffer, "gui_buffers");
+    while (ptr_buffer)
+    {
+        ptr_name = weechat_buffer_get_string (ptr_buffer, "localvar_relay_remote");
+        if (ptr_name && (weechat_strcmp (ptr_name, remote->name) == 0))
+        {
+            weechat_buffer_clear (ptr_buffer);
+            weechat_nicklist_remove_all (ptr_buffer);
+        }
+        ptr_buffer = weechat_hdata_move (relay_hdata_buffer, ptr_buffer, 1);
+    }
+}
+
 /*
  * Callback for response to GET /api/version.
  */
@@ -810,7 +1112,7 @@ RELAY_REMOTE_EVENT_CALLBACK(version)
     const char *weechat_version, *weechat_version_git, *relay_api_version;
     char *weechat_version_local, request[1024];
 
-    if (!event->json)
+    if (!event || !event->json)
         return WEECHAT_RC_OK;
 
     JSON_GET_STR(event->json, weechat_version);
@@ -845,15 +1147,16 @@ RELAY_REMOTE_EVENT_CALLBACK(version)
                 relay_remote_network_disconnect (event->remote);
                 return WEECHAT_RC_OK;
         }
-
+        relay_remote_event_clear_buffers (event->remote);
         event->remote->version_ok = 1;
-        snprintf (request, sizeof (request),
-                  "{\"request\": \"GET /api/buffers?"
-                  "lines=-%d"
-                  "&nicks=true"
-                  "&colors=weechat"
-                  "\"}",
-                  weechat_config_integer (relay_config_api_remote_get_lines));
+        snprintf (
+            request, sizeof (request),
+            "{"
+            "\"request\": "
+            "\"GET /api/buffers?lines=-%d&nicks=true&colors=weechat\", "
+            "\"request_id\": \"" RELAY_REMOTE_EVENT_ID_INITIAL_SYNC "\""
+            "}",
+            weechat_config_integer (relay_config_api_remote_get_lines));
         relay_remote_network_send (event->remote, RELAY_MSG_STANDARD,
                                    request, strlen (request));
     }
@@ -861,6 +1164,32 @@ RELAY_REMOTE_EVENT_CALLBACK(version)
     return WEECHAT_RC_OK;
 }
 
+/*
+ * Callback for events "upgrade" and "upgrade_ended".
+ */
+
+RELAY_REMOTE_EVENT_CALLBACK(upgrade)
+{
+    if (!event || !event->remote)
+        return WEECHAT_RC_OK;
+
+    relay_remote_network_disconnect (event->remote);
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Callback for event "quit".
+ */
+
+RELAY_REMOTE_EVENT_CALLBACK(quit)
+{
+    if (!event || !event->remote)
+        return WEECHAT_RC_OK;
+
+    relay_remote_network_disconnect (event->remote);
+    return WEECHAT_RC_OK;
+}
+
 /*
  * Synchronizes with remote.
  */
@@ -902,8 +1231,8 @@ end:
 void
 relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
 {
-    cJSON *json, *json_body, *json_event, *json_obj;
-    const char *body_type, *name;
+    cJSON *json, *json_body, *json_obj;
+    const char *body_type, *event_name, *request_id;
     long long buffer_id;
     int i, rc, code;
     struct t_relay_remote_event_cb event_cb[] = {
@@ -917,6 +1246,8 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
         { "input_*", &relay_remote_event_cb_input },
         { "nicklist_group_*", &relay_remote_event_cb_nick_group },
         { "nicklist_nick_*", &relay_remote_event_cb_nick },
+        { "upgrade*", &relay_remote_event_cb_upgrade },
+        { "quit", &relay_remote_event_cb_quit },
         { NULL, NULL },
     };
     t_relay_remote_event_func *callback;
@@ -944,28 +1275,30 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
 
     JSON_GET_NUM(json, code, -1);
     JSON_GET_STR(json, body_type);
-    json_event = cJSON_GetObjectItem (json, "event");
+    JSON_GET_STR(json, request_id);
     json_body = cJSON_GetObjectItem (json, "body");
 
     if (!body_type && ((code == 200) || (code == 204)))
         return;
 
-    if (json_event && cJSON_IsObject (json_event))
-    {
-        JSON_GET_STR(json_event, name);
-        event.name = name;
-        JSON_GET_NUM(json_event, buffer_id, -1);
-        event.buffer = relay_remote_event_search_buffer (remote, buffer_id);
-    }
+    JSON_GET_STR(json, event_name);
+    event.name = event_name;
+    JSON_GET_NUM(json, buffer_id, -1);
+    event.buffer = relay_remote_event_search_buffer (remote, buffer_id);
 
     callback = NULL;
 
     if (code == 200)
     {
-        if (weechat_strcmp (body_type, "buffer") == 0)
+        if ((weechat_strcmp (body_type, "buffers") == 0)
+            || (weechat_strcmp (body_type, "buffer") == 0))
+        {
             callback = &relay_remote_event_cb_buffer;
+        }
         else if (weechat_strcmp (body_type, "version") == 0)
+        {
             callback = &relay_remote_event_cb_version;
+        }
     }
     else if (event.name)
     {
@@ -981,6 +1314,13 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
 
     if (callback)
     {
+        event.json = json_body;
+        if ((weechat_strcmp (body_type, "buffers") == 0)
+            && (weechat_strcmp (request_id,
+                                RELAY_REMOTE_EVENT_ID_INITIAL_SYNC) == 0))
+        {
+            relay_remote_event_initial_sync_buffers (&event);
+        }
         rc = WEECHAT_RC_OK;
         if (cJSON_IsArray (json_body))
         {
@@ -992,7 +1332,6 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
         }
         else
         {
-            event.json = json_body;
             rc = (callback) (&event);
         }
         if (rc == WEECHAT_RC_ERROR)
@@ -1001,7 +1340,8 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
 
     if (!remote->synced
         && (code == 200)
-        && (weechat_strcmp (body_type, "buffer") == 0))
+        && ((weechat_strcmp (body_type, "buffers") == 0)
+            || (weechat_strcmp (body_type, "buffer") == 0)))
     {
         relay_remote_event_sync_with_remote (remote);
     }
@@ -1009,18 +1349,20 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
     return;
 
 error_data:
-    weechat_printf (NULL,
-                    "%sremote[%s]: invalid data received from remote: \"%s\"",
-                    weechat_prefix ("error"),
-                    remote->name,
-                    data);
+    weechat_printf (
+        NULL,
+        _("%sremote[%s]: invalid data received from remote relay: \"%s\""),
+        weechat_prefix ("error"),
+        remote->name,
+        data);
     return;
 
 error_cb:
-    weechat_printf (NULL,
-                    "%sremote[%s]: callback failed for body type \"%s\"",
-                    weechat_prefix ("error"),
-                    remote->name,
-                    body_type);
+    weechat_printf (
+        NULL,
+        _("%sremote[%s]: callback failed for body type \"%s\""),
+        weechat_prefix ("error"),
+        remote->name,
+        body_type);
     return;
 }
