@@ -64,7 +64,7 @@ trigger_command_display_trigger_internal (const char *name,
                                           int verbose)
 {
     char str_conditions[64], str_regex[64], str_command[64], str_rc[64];
-    char str_post_action[64], spaces[256];
+    char str_post_action[64], spaces[256], **ptr_command;
     int i, length;
 
     if (verbose >= 1)
@@ -133,7 +133,7 @@ trigger_command_display_trigger_internal (const char *name,
         }
         if (commands)
         {
-            for (i = 0; commands[i]; i++)
+            for (ptr_command = commands, i = 0; *ptr_command; ptr_command++, i++)
             {
                 weechat_printf_date_tags (
                     NULL, 0, "no_trigger",
@@ -143,7 +143,7 @@ trigger_command_display_trigger_internal (const char *name,
                     i + 1,
                     weechat_color ("chat_delimiters"),
                     weechat_color ("reset"),
-                    commands[i],
+                    *ptr_command,
                     weechat_color ("chat_delimiters"));
             }
         }
@@ -436,8 +436,7 @@ trigger_command_list_default (int verbose)
     }
 
     trigger_regex_free (&regex_count, &regex);
-    if (commands)
-        weechat_string_free_split (commands);
+    weechat_string_free_split (commands);
 }
 
 /*
@@ -568,10 +567,8 @@ trigger_command_rename (struct t_trigger *trigger, const char *new_name)
     }
 
 end:
-    if (name)
-        free (name);
-    if (name2)
-        free (name2);
+    free (name);
+    free (name2);
 }
 
 /*
@@ -773,16 +770,6 @@ trigger_command_trigger (const void *pointer, void *data,
                 goto end;
             }
         }
-        ptr_trigger = trigger_alloc (sargv[0]);
-        if (!ptr_trigger)
-        {
-            weechat_printf_date_tags (
-                NULL, 0, "no_trigger",
-                _("%s%s: failed to create trigger \"%s\""),
-                weechat_prefix ("error"), TRIGGER_PLUGIN_NAME,
-                sargv[0]);
-            goto end;
-        }
         ptr_trigger = trigger_new (
             sargv[0],                      /* name */
             (weechat_strcmp (argv[1], "addoff") == 0) ? "off" : "on",
@@ -850,8 +837,7 @@ trigger_command_trigger (const void *pointer, void *data,
             weechat_buffer_set (buffer, "input_pos", "13");
             free (input);
         }
-        if (items)
-            weechat_string_free_split (items);
+        weechat_string_free_split (items);
         goto end;
     }
 
@@ -902,18 +888,12 @@ trigger_command_trigger (const void *pointer, void *data,
             arg_command,
             arg_return_code,
             arg_post_action);
-        if (arg_arguments)
-            free (arg_arguments);
-        if (arg_conditions)
-            free (arg_conditions);
-        if (arg_regex)
-            free (arg_regex);
-        if (arg_command)
-            free (arg_command);
-        if (arg_return_code)
-            free (arg_return_code);
-        if (arg_post_action)
-            free (arg_post_action);
+        free (arg_arguments);
+        free (arg_conditions);
+        free (arg_regex);
+        free (arg_command);
+        free (arg_return_code);
+        free (arg_post_action);
         if (input)
         {
             if (weechat_strcmp (argv[1], "output") == 0)
@@ -1131,8 +1111,7 @@ trigger_command_trigger (const void *pointer, void *data,
                         weechat_printf_date_tags (NULL, 0, "no_trigger",
                                                   _("Trigger \"%s\" removed"),
                                                   name);
-                        if (name)
-                            free (name);
+                        free (name);
                     }
                 }
                 ptr_trigger = ptr_next_trigger;
@@ -1182,8 +1161,7 @@ trigger_command_trigger (const void *pointer, void *data,
                     }
                     else
                     {
-                        if (ptr_trigger)
-                            trigger_free (ptr_trigger);
+                        trigger_free (ptr_trigger);
                         ptr_trigger = trigger_new (
                             trigger_config_default_list[j][0],   /* name */
                             trigger_config_default_list[j][1],   /* enabled */
@@ -1261,8 +1239,7 @@ error:
     rc = WEECHAT_RC_ERROR;
 
 end:
-    if (sargv)
-        weechat_string_free_split (sargv);
+    weechat_string_free_split (sargv);
 
     if (rc == WEECHAT_RC_ERROR)
         WEECHAT_COMMAND_ERROR;
