@@ -1,7 +1,7 @@
 /*
  * test-core-infolist.cpp - test infolist functions
  *
- * Copyright (C) 2014-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -24,8 +24,8 @@
 
 extern "C"
 {
-#include "src/core/wee-hook.h"
-#include "src/core/wee-infolist.h"
+#include "src/core/core-hook.h"
+#include "src/core/core-infolist.h"
 }
 
 struct t_hook *hook_test_infolist = NULL;
@@ -253,6 +253,9 @@ TEST(CoreInfolist, Valid)
     infolist_free (infolist);
 
     LONGS_EQUAL(0, infolist_valid (infolist));
+
+    /* test free of NULL infolist */
+    infolist_free (NULL);
 }
 
 /*
