@@ -1,7 +1,7 @@
 /*
  * test-gui-nick.cpp - test nick functions
  *
- * Copyright (C) 2019-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2019-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -23,8 +23,8 @@
 
 extern "C"
 {
-#include "src/core/wee-config.h"
-#include "src/core/wee-string.h"
+#include "src/core/core-config.h"
+#include "src/core/core-string.h"
 #include "src/gui/gui-color.h"
 #include "src/gui/gui-nick.h"
 
