@@ -1,7 +1,7 @@
 /*
  * test-irc-server.cpp - test IRC protocol functions
  *
- * Copyright (C) 2020-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2020-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -27,7 +27,7 @@ extern "C"
 {
 #include <stdio.h>
 #include <string.h>
-#include "src/core/wee-config-file.h"
+#include "src/core/core-config-file.h"
 #include "src/plugins/plugin.h"
 #include "src/plugins/irc/irc-channel.h"
 #include "src/plugins/irc/irc-server.h"
@@ -1118,6 +1118,56 @@ TEST(IrcServer, GetTagsToSend)
     /* TODO: write tests */
 }
 
+/*
+ * Tests functions:
+ *   irc_server_outqueue_all_empty
+ */
+
+TEST(IrcServer, OutqueueAllEmpty)
+{
+    /* TODO: write tests */
+}
+
+/*
+ * Tests functions:
+ *   irc_server_outqueue_timer_cb
+ */
+
+TEST(IrcServer, OutqueueTimerCb)
+{
+    /* TODO: write tests */
+}
+
+/*
+ * Tests functions:
+ *   irc_server_outqueue_timer_remove
+ */
+
+TEST(IrcServer, OutqueueTimerRemove)
+{
+    /* TODO: write tests */
+}
+
+/*
+ * Tests functions:
+ *   irc_server_outqueue_timer_add
+ */
+
+TEST(IrcServer, OutqueueTimerAdd)
+{
+    /* TODO: write tests */
+}
+
+/*
+ * Tests functions:
+ *   irc_server_outqueue_send_one_msg
+ */
+
+TEST(IrcServer, OutqueueSendOneMsg)
+{
+    /* TODO: write tests */
+}
+
 /*
  * Tests functions:
  *   irc_server_outqueue_send
@@ -1428,6 +1478,16 @@ TEST(IrcServer, DisconnectAll)
     /* TODO: write tests */
 }
 
+/*
+ * Tests functions:
+ *   irc_server_execute_command
+ */
+
+TEST(IrcServer, ExecuteCommand)
+{
+    /* TODO: write tests */
+}
+
 /*
  * Tests functions:
  *   irc_server_autojoin_create_buffers
