@@ -146,10 +146,17 @@ TEST(RelayApiMsg, BufferToJson)
     WEE_CHECK_OBJ_STR("weechat", json, "short_name");
     WEE_CHECK_OBJ_NUM(1, json, "number");
     WEE_CHECK_OBJ_STR("formatted", json, "type");
+    WEE_CHECK_OBJ_BOOL(0, json, "hidden");
     WEE_CHECK_OBJ_STRN("WeeChat", 7, json, "title");
+    WEE_CHECK_OBJ_STR("", json, "modes");
+    WEE_CHECK_OBJ_STR("", json, "input_prompt");
+    WEE_CHECK_OBJ_STR("", json, "input");
+    WEE_CHECK_OBJ_NUM(0, json, "input_position");
+    WEE_CHECK_OBJ_BOOL(0, json, "input_multiline");
     WEE_CHECK_OBJ_BOOL(0, json, "nicklist");
     WEE_CHECK_OBJ_BOOL(0, json, "nicklist_case_sensitive");
     WEE_CHECK_OBJ_BOOL(1, json, "nicklist_display_groups");
+    WEE_CHECK_OBJ_BOOL(1, json, "time_displayed");
     json_local_vars = cJSON_GetObjectItem (json, "local_variables");
     CHECK(json_local_vars);
     CHECK(cJSON_IsObject (json_local_vars));
@@ -170,6 +177,19 @@ TEST(RelayApiMsg, BufferToJson)
     POINTERS_EQUAL(NULL, cJSON_GetObjectItem (json, "nicks"));
     cJSON_Delete (json);
 
+    gui_buffer_hide (gui_buffers);
+    gui_buffer_set_time_for_each_line (gui_buffers, 0);
+
+    json = relay_api_msg_buffer_to_json (gui_buffers, 0L, 0L, 0, RELAY_API_COLORS_ANSI);
+    CHECK(json);
+    CHECK(cJSON_IsObject (json));
+    WEE_CHECK_OBJ_BOOL(1, json, "hidden");
+    WEE_CHECK_OBJ_BOOL(0, json, "time_displayed");
+    cJSON_Delete (json);
+
+    gui_buffer_unhide (gui_buffers);
+    gui_buffer_set_time_for_each_line (gui_buffers, 1);
+
     /* buffer with 2 lines, without nicks */
     json = relay_api_msg_buffer_to_json (gui_buffers, 2L, 0L, 0, RELAY_API_COLORS_ANSI);
     CHECK(json);
